# Copyright 2023 Tyler Schlichenmeyer

# This file is part of PsychoAnalyze.
# PsychoAnalyze is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.

# PsychoAnalyze is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# PsychoAnalyze. If not, see <https://www.gnu.org/licenses/>.

"""Top-level package for `psychoanalyze`.

Modules:

- [`plot`][psychoanalyze.plot]: Global plot settings and generic plot utilities.
- [`sigmoids`][psychoanalyze.sigmoids]: Implementations of psychometric sigmoid
functions.

Subpackages:

- [`data`][psychoanalyze.data]: Submodules in [`psychoanalyze.data`][psychoanalyze.data]
contain data manipulation and transformation functions that relate to a level in the
data hierarchy.
- [`analysis`][psychoanalyze.analysis]: Submodules in
[`psychoanalyze.analysis`][psychoanalyze.analysis] contain data manipulation and
transformation functions, often corresponding to a level in the data hierarchy.
"""
